@@ -9,6 +9,7 @@
 #ifndef XCONTAINER_HPP
 #define XCONTAINER_HPP
 
+#include <algorithm>
 #include <functional>
 #include <numeric>
 #include <stdexcept>
@@ -84,7 +85,7 @@ namespace xt
 
         size_type size() const noexcept;
 
-        size_type dimension() const noexcept;
+        constexpr size_type dimension() const noexcept;
 
         const inner_shape_type& shape() const noexcept;
         const inner_strides_type& strides() const noexcept;
@@ -148,11 +149,8 @@ namespace xt
         using container_iterator = typename container_type::iterator;
         using const_container_iterator = typename container_type::const_iterator;
 
-        container_iterator data_xbegin() noexcept;
-        const_container_iterator data_xbegin() const noexcept;
-
-        container_iterator data_xend(layout_type l) noexcept;
-        const_container_iterator data_xend(layout_type l) const noexcept;
+        reference data_element(size_type i);
+        const_reference data_element(size_type i) const;
 
     protected:
 
@@ -165,8 +163,16 @@ namespace xt
         xcontainer(xcontainer&&) = default;
         xcontainer& operator=(xcontainer&&) = default;
 
+        container_iterator data_xbegin() noexcept;
+        const_container_iterator data_xbegin() const noexcept;
+        container_iterator data_xend(layout_type l) noexcept;
+        const_container_iterator data_xend(layout_type l) const noexcept;
+
     private:
 
+        template <class C>
+        friend class xstepper;
+
         template <class It>
         It data_xend_impl(It end, layout_type l) const noexcept;
 
@@ -267,6 +273,7 @@ namespace xt
         else
         {
             auto leading_stride = (l == layout_type::row_major ? strides().back() : strides().front());
+            leading_stride = std::max(leading_stride, typename inner_strides_type::value_type(1));
             return end - 1 + leading_stride;
         }
     }
@@ -318,7 +325,7 @@ namespace xt
      * Returns the number of dimensions of the container.
      */
     template <class D>
-    inline auto xcontainer<D>::dimension() const noexcept -> size_type
+    inline constexpr auto xcontainer<D>::dimension() const noexcept -> size_type
     {
         return shape().size();
     }
@@ -426,7 +433,7 @@ namespace xt
      * Returns a reference to the element at the specified position in the container.
      * @param first iterator starting the sequence of indices
      * @param last iterator ending the sequence of indices
-     * The number of indices in the squence should be equal to or greater
+     * The number of indices in the sequence should be equal to or greater
      * than the number of dimensions of the container.
      */
     template <class D>
@@ -441,7 +448,7 @@ namespace xt
      * Returns a reference to the element at the specified position in the container.
      * @param first iterator starting the sequence of indices
      * @param last iterator ending the sequence of indices
-     * The number of indices in the squence should be equal to or greater
+     * The number of indices in the sequence should be equal to or greater
      * than the number of dimensions of the container.
      */
     template <class D>
@@ -720,6 +727,18 @@ namespace xt
         return data_xend_impl(data().end(), l);
     }
 
+    template <class D>
+    inline auto xcontainer<D>::data_element(size_type i) -> reference
+    {
+        return data()[i];
+    }
+
+    template <class D>
+    inline auto xcontainer<D>::data_element(size_type i) const -> const_reference
+    {
+        return data()[i];
+    }
+
     /*************************************
      * xstrided_container implementation *
      *************************************/
